#include <jni.h>

#include <GLES2/gl2.h>
#include <GLES2/gl2ext.h>

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

static const char vertexShader[] = 
  "attribute vec4 position;       \n"
  "void main(void) {              \n"
  "  gl_Position = position;      \n"
  "}                              \n";

static const char fragmentShader[] =
  "precision mediump float;                  \n"
  "void main(void) {                                \n"
  "  gl_FragColor = vec4(0.0, 1.0, 0.0, 1.0);   \n"
  "}                                                \n";


uint program;
uint position;
uint color;


uint loadShader(GLenum shaderType, const char* pSource) {
    uint shader = glCreateShader(shaderType);
    if (shader) {
        glShaderSource(shader, 1, &pSource, NULL);
        glCompileShader(shader);
        int compiled = 0;
        glGetShaderiv(shader, GL_COMPILE_STATUS, &compiled);
        if (!compiled) {
            int infoLen = 0;
            glGetShaderiv(shader, GL_INFO_LOG_LENGTH, &infoLen);
            if (infoLen) {
                char* buf = (char*) malloc(infoLen);
                if (buf) {
                    glGetShaderInfoLog(shader, infoLen, NULL, buf);
                    free(buf);
                }
                glDeleteShader(shader);
                shader = 0;
            }
        }
    }
    return shader;
}

GLuint createProgram(const char* pVertexSource, const char* pFragmentSource) {
    GLuint vertexShader = loadShader(GL_VERTEX_SHADER, pVertexSource);
    if (!vertexShader) {
        return 0;
    }

    GLuint pixelShader = loadShader(GL_FRAGMENT_SHADER, pFragmentSource);
    if (!pixelShader) {
        return 0;
    }

    GLuint program = glCreateProgram();

    if (program) {
      glAttachShader(program, vertexShader);
      glAttachShader(program, pixelShader);
      glLinkProgram(program);
      GLint linkStatus = GL_FALSE;
      glGetProgramiv(program, GL_LINK_STATUS, &linkStatus);
      if (linkStatus != GL_TRUE) {
	GLint bufLength = 0;
	glGetProgramiv(program, GL_INFO_LOG_LENGTH, &bufLength);
	if (bufLength) {
	  char* buf = (char*) malloc(bufLength);
	  if (buf) {
	    glGetProgramInfoLog(program, bufLength, NULL, buf);
	    free(buf);
	  }
	}
	glDeleteProgram(program);
	program = 0;
      }
    }

    return program;

}



const GLfloat triangles[] = { 0.0f, 0.5f,
			      -0.5f, -0.5f,
			      0.5f, -0.5f };

const float colors[] = {1.0f, 0.0f, 0.0f, 1.0f,
			0.0f, 1.0f, 0.0f, 1.0f,
			0.0f, 0.0f, 1.0f, 1.0f};

JNIEXPORT void JNICALL Java_com_example_myandroid_GLActivity_init(JNIEnv * env, jobject obj,  jint width, jint height)
{

    program = createProgram(vertexShader, fragmentShader);
    position = glGetAttribLocation(program, "position");
    //color = glGetAttribLocation(program, "color");

    glViewport(0, 0, width, height);

}


JNIEXPORT void JNICALL Java_com_example_myandroid_GLActivity_step(JNIEnv * env, jobject obj)
{

    glClearColor(0.0f, 0, 0, 1.0f);
    glClear( GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);

    glUseProgram(program);

    glVertexAttribPointer(position, 2, GL_FLOAT, GL_FALSE, 0, triangles);
    //glVertexAttribPointer(color, 4, GL_FLOAT, GL_FALSE, 0, colors);

    glEnableVertexAttribArray(position);
    //glEnableVertexAttribArray(color);

    glDrawArrays(GL_TRIANGLES, 0, 3);

}

